/*
 * Copyright (c) 2018 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

/** @file
 *  @brief Nordic UART Bridge Service (NUS) sample
 */
#include "uart_async_adapter.h"

#include <zephyr/types.h>
#include <zephyr/kernel.h>
#include <zephyr/drivers/uart.h>
#include <zephyr/usb/usb_device.h>

#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <soc.h>

#include <zephyr/bluetooth/bluetooth.h>
#include <zephyr/bluetooth/uuid.h>
#include <zephyr/bluetooth/gatt.h>
#include <zephyr/bluetooth/hci.h>

#include <bluetooth/services/nus.h>

#include <dk_buttons_and_leds.h>

#include <zephyr/settings/settings.h>

#include <stdio.h>

/////
#include <zephyr/logging/log.h>
#include <zephyr/drivers/gpio.h>
#include <hal/nrf_gpio.h>
#include <zephyr/sys/printk.h>
/////


/////
//#define nRF21540DK 2540	

//#define BT840 840
//#define BT840X 8402		//52840+66112
//#define BT840XEE 8405		//52840+66112
//#define BT840N_V3 8403	//52840+21540
//#define BT840N_V4 8404	//52840+21540
//#define BM840 8406
//#define BM840P 8407

#define BT832 832		
//#define BT832X 8322		//52832+66112

//#define BT833 833

//#define BT40 400			
//#define BT40N_V6 406		//5340+21540

#ifdef BT840X
	#define APP_CHL_PIN	8
	#define APP_CPS_PIN	6 
#elif BT840XEE
	#define APP_CHL_PIN	8
	#define APP_CPS_PIN	6 
#elif BT832X
	#define APP_CPS_PIN	6 
#endif

#ifdef BT40
#elif BT832
#elif BT40N_V6
	#define DCDC_VOLTAGE
#else
	#define MONITOR_TXPOWER_SETTING		250
#endif

#define Factory_TC
#ifdef Factory_TC
	const struct device *led_dev;
	const struct device *led_dev1;
	static bool is_connect = false;
	#define BLE_ADV_NAME
	#ifdef BLE_ADV_NAME
		static bool command_pending = false;
		static char name_tH = '?';
		static char name_tL = '?';
		static char name_mH = '?';
		static char name_mL = '?';
	#endif
	static uint8_t bnt1_val;
	static uint8_t bnt2_val;
	static uint8_t bnt3_val;
	static uint8_t bnt4_val;
	static bool bnt_flag = false;
#endif
/////

#define LOG_MODULE_NAME peripheral_uart
LOG_MODULE_REGISTER(LOG_MODULE_NAME);

#define STACKSIZE CONFIG_BT_NUS_THREAD_STACK_SIZE
#define PRIORITY 7

#define DEVICE_NAME CONFIG_BT_DEVICE_NAME
#define DEVICE_NAME_LEN	(sizeof(DEVICE_NAME) - 1)

#define RUN_STATUS_LED DK_LED1
#define RUN_LED_BLINK_INTERVAL 1000

#define CON_STATUS_LED DK_LED2

#define KEY_PASSKEY_ACCEPT DK_BTN1_MSK
#define KEY_PASSKEY_REJECT DK_BTN2_MSK

#define UART_BUF_SIZE CONFIG_BT_NUS_UART_BUFFER_SIZE
#define UART_WAIT_FOR_BUF_DELAY K_MSEC(50)
#define UART_WAIT_FOR_RX CONFIG_BT_NUS_UART_RX_WAIT_TIME

static K_SEM_DEFINE(ble_init_ok, 0, 1);

static struct bt_conn *current_conn;
static struct bt_conn *auth_conn;

static const struct device *uart = DEVICE_DT_GET(DT_CHOSEN(nordic_nus_uart));
static struct k_work_delayable uart_work;

struct uart_data_t {
	void *fifo_reserved;
	uint8_t data[UART_BUF_SIZE];
	uint16_t len;
};

static K_FIFO_DEFINE(fifo_uart_tx_data);
static K_FIFO_DEFINE(fifo_uart_rx_data);

/////
#ifdef BLE_ADV_NAME
static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR))
};
#else
static const struct bt_data ad[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	BT_DATA(BT_DATA_NAME_COMPLETE, DEVICE_NAME, DEVICE_NAME_LEN),
};
#endif
/////

static const struct bt_data sd[] = {
	BT_DATA_BYTES(BT_DATA_UUID128_ALL, BT_UUID_NUS_VAL),
};

#if CONFIG_BT_NUS_UART_ASYNC_ADAPTER
UART_ASYNC_ADAPTER_INST_DEFINE(async_adapter);
#else
static const struct device *const async_adapter;
#endif

/////
static void button_changed(uint32_t button_state, uint32_t has_changed);
/////

//////////
#ifdef Factory_TC
void All_IO_Output(){

#ifdef BT40
#elif BT40N_V6
#elif BT840X
#elif BT832X
#elif BT840XEE
#else
	nrf_gpio_cfg_output(0);		//clock
	nrf_gpio_cfg_output(1);		//clock
#endif
	nrf_gpio_cfg_output(2);		
	nrf_gpio_cfg_output(3);	
	nrf_gpio_cfg_output(4);		
	nrf_gpio_cfg_output(5);
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_cfg_output(6);	
#endif
    nrf_gpio_cfg_output(7);	
	nrf_gpio_cfg_output(8);		
	nrf_gpio_cfg_output(9);	
	nrf_gpio_cfg_output(10);
#ifdef BT832
#elif BT832X
#elif BM840
#elif BM840P
#else
	nrf_gpio_cfg_output(11);		
	nrf_gpio_cfg_output(12);
#endif
    nrf_gpio_cfg_output(13);
	nrf_gpio_cfg_output(14);
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_cfg_output(15);
	nrf_gpio_cfg_output(16);
#endif
	nrf_gpio_cfg_output(17);
#ifdef BT840
#elif BT840X
#elif BT840N_V4
#elif BT840XEE
#elif BT833
#elif BM840
#elif BM840P
#else
	nrf_gpio_cfg_output(18);
#endif
	nrf_gpio_cfg_output(19);
#ifdef BT40
#elif BT40N_V6
#else
	nrf_gpio_cfg_output(20);
#endif
#ifdef BT832
#elif BT832X
#else
	nrf_gpio_cfg_output(21);
#endif
#ifdef BT40
#elif BT40N_V6
#else
	nrf_gpio_cfg_output(22);
#endif
    nrf_gpio_cfg_output(23);
	nrf_gpio_cfg_output(24);
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_cfg_output(25);
	nrf_gpio_cfg_output(26);
	nrf_gpio_cfg_output(27);
#endif
	nrf_gpio_cfg_output(28);
	nrf_gpio_cfg_output(29);
	nrf_gpio_cfg_output(30);
	nrf_gpio_cfg_output(31);
	nrf_gpio_cfg_output(32);		//P1.00
#ifdef BT840
#elif BT840X
#elif BT840N_V4
#elif BT840XEE
#elif BM840
#elif BM840P
#else
    nrf_gpio_cfg_output(33);		//P1.01
	nrf_gpio_cfg_output(34);		//P1.02
#endif
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_cfg_output(35);		//P1.03
	nrf_gpio_cfg_output(36);		//P1.04
	nrf_gpio_cfg_output(37);		//P1.05
	nrf_gpio_cfg_output(38);		//P1.06
	nrf_gpio_cfg_output(39);		//P1.07
#endif
	nrf_gpio_cfg_output(40);		//P1.08
	nrf_gpio_cfg_output(41);		//P1.09
#ifdef BM840
#elif BM840P
#else
    nrf_gpio_cfg_output(42);		//P1.10
	nrf_gpio_cfg_output(43);		//P1.11
	nrf_gpio_cfg_output(44);		//P1.12
	nrf_gpio_cfg_output(45);		//P1.13
	nrf_gpio_cfg_output(46);		//P1.14
#endif
    nrf_gpio_cfg_output(47);		//P1.15
}

void All_IO_low(){

	All_IO_Output();

#ifdef BT40
#elif BT40N_V6
#elif BT840X
#elif BT832X
#elif BT840XEE
#else
	nrf_gpio_pin_clear(0);
	nrf_gpio_pin_clear(1);
#endif
	nrf_gpio_pin_clear(2);
	nrf_gpio_pin_clear(3);
	nrf_gpio_pin_clear(4);
	nrf_gpio_pin_clear(5);
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_pin_clear(6);
#endif
	nrf_gpio_pin_clear(7);
	nrf_gpio_pin_clear(8);
	nrf_gpio_pin_clear(9);
	nrf_gpio_pin_clear(10);
#ifdef BT832
#elif BT832X
#elif BM840
#elif BM840P
#else
	nrf_gpio_pin_clear(11);
	nrf_gpio_pin_clear(12);
#endif
	nrf_gpio_pin_clear(13);
	nrf_gpio_pin_clear(14);
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_pin_clear(15);
	nrf_gpio_pin_clear(16);
#endif

	nrf_gpio_pin_clear(17);
#ifdef BT840
#elif BT840X
#elif BT840N_V4
#elif BT840XEE
#elif BM840
#elif BM840P
#else
	nrf_gpio_pin_clear(18);
#endif
	nrf_gpio_pin_clear(19);
#ifdef BT40
#elif BT40N_V6
#else
	nrf_gpio_pin_clear(20);
#endif
#ifdef BT832
#elif BT832X
#else
	nrf_gpio_pin_clear(21);
#endif
#ifdef BT40
#elif BT40N_V6
#else
	nrf_gpio_pin_clear(22);
#endif
	nrf_gpio_pin_clear(23);
	nrf_gpio_pin_clear(24);
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_pin_clear(25);
	nrf_gpio_pin_clear(26);
	nrf_gpio_pin_clear(27);
#endif
	nrf_gpio_pin_clear(28);
	nrf_gpio_pin_clear(29);
	nrf_gpio_pin_clear(30);
	nrf_gpio_pin_clear(31);
	nrf_gpio_pin_clear(32);			//P1.00
#ifdef BT840
#elif BT840X
#elif BT840N_V4
#elif BT840XEE
#elif BT833
#elif BM840
#elif BM840P
#else
	nrf_gpio_pin_clear(33);			//P1.01
	nrf_gpio_pin_clear(34);			//P1.02
#endif
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_pin_clear(35);			//P1.03
	nrf_gpio_pin_clear(36);			//P1.04
	nrf_gpio_pin_clear(37);			//P1.05
	nrf_gpio_pin_clear(38);			//P1.06
	nrf_gpio_pin_clear(39);			//P1.07
#endif
	nrf_gpio_pin_clear(40);			//P1.08
	nrf_gpio_pin_clear(41);			//P1.09
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_pin_clear(42);			//P1.10
	nrf_gpio_pin_clear(43);			//P1.11
	nrf_gpio_pin_clear(44);			//P1.12
	nrf_gpio_pin_clear(45);			//P1.13
	nrf_gpio_pin_clear(46);			//P1.14
#endif
	nrf_gpio_pin_clear(47);			//P1.15
}

void All_GPIO_High(){

#ifdef BT40
#elif BT40N_V6
#elif BT840X
#elif BT832X
#elif BT840XEE
#else
	nrf_gpio_pin_set(0);	
	nrf_gpio_pin_set(1);
#endif

	nrf_gpio_pin_set(2);
	nrf_gpio_pin_set(3);
	nrf_gpio_pin_set(4);
	nrf_gpio_pin_set(5);
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_pin_set(6);
#endif

	nrf_gpio_pin_set(7);
	nrf_gpio_pin_set(8);
	nrf_gpio_pin_set(9);
	nrf_gpio_pin_set(10);
#ifdef BT832
#elif BT832X
#elif BM840
#elif BM840P
#else

	nrf_gpio_pin_set(11);
	nrf_gpio_pin_set(12);
#endif
	nrf_gpio_pin_set(13);
	nrf_gpio_pin_set(14);
#ifdef BM840
#elif BM840P
#else

	nrf_gpio_pin_set(15);
	nrf_gpio_pin_set(16);
#endif

	nrf_gpio_pin_set(17);
#ifdef BT840
#elif BT840X
#elif BT840N_V4
#elif BT840XEE
#elif BT833
#elif BM840
#elif BM840P
#else
	nrf_gpio_pin_set(18);
#endif
	nrf_gpio_pin_set(19);
#ifdef BT40
#elif BT40N_V6
#else
	nrf_gpio_pin_set(20);
#endif
#ifdef BT832
#elif BT832X
#else
	nrf_gpio_pin_set(21);
#endif
#ifdef BT40
#elif BT40N_V6
#else
	nrf_gpio_pin_set(22);
#endif
	nrf_gpio_pin_set(23);
	nrf_gpio_pin_set(24);
#ifdef BM840
#elif BM840P
#else
	nrf_gpio_pin_set(25);
	nrf_gpio_pin_set(26);
	nrf_gpio_pin_set(27);
#endif
	nrf_gpio_pin_set(28);
	nrf_gpio_pin_set(29);
	nrf_gpio_pin_set(30);
	nrf_gpio_pin_set(31);
	nrf_gpio_pin_set(32);			//P1.00
#ifdef BT840
#elif BT840X
#elif BT840N_V4
#elif BT840XEE
#elif BT833
#elif BM840
#elif BM840P
#else
	nrf_gpio_pin_set(33);			//P1.01
	nrf_gpio_pin_set(34);			//P1.02
#endif
#ifdef BM840
#elif BM840P
#else

	nrf_gpio_pin_set(35);			//P1.03
	nrf_gpio_pin_set(36);			//P1.04
	nrf_gpio_pin_set(37);			//P1.05
	nrf_gpio_pin_set(38);			//P1.06
	nrf_gpio_pin_set(39);			//P1.07
#endif

	nrf_gpio_pin_set(40);			//P1.08
	nrf_gpio_pin_set(41);			//P1.09
#ifdef BM840
#elif BM840P
#else

	nrf_gpio_pin_set(42);			//P1.10
	nrf_gpio_pin_set(43);			//P1.11
	nrf_gpio_pin_set(44);			//P1.12
	nrf_gpio_pin_set(45);			//P1.13
	nrf_gpio_pin_set(46);			//P1.14
#endif

	nrf_gpio_pin_set(47);			//P1.15
}

void Factory_Test(uint8_t which){	

	All_IO_low();

	if(which<0x10){
		LOG_INF("Testing:0x0%x\n",which);
		//printk("Testing:0x0%x\n",which);
	}else{
		LOG_INF("Testing:0x%x\n",which);
		//printk("Testing:0x%x\n",which);
	}	

	switch(which){
		case 0x00:
			nrf_gpio_pin_set(0);
			LOG_INF("nrf_gpio_pin_set(0)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(0)\n");
			break;
		case 0x01:
			nrf_gpio_pin_set(1);
			LOG_INF("nrf_gpio_pin_set(1)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(1)\n");
			break;
		case 0x02:
			nrf_gpio_pin_set(2);
			LOG_INF("nrf_gpio_pin_set(2)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(2)\n");
			break;		
		case 0x03:
			nrf_gpio_pin_set(3);
			LOG_INF("nrf_gpio_pin_set(3)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(3)\n");
			break;						
		case 0x04:
			nrf_gpio_pin_set(4);
			LOG_INF("nrf_gpio_pin_set(4)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(4)\n");
			break;						
		case 0x05:
			nrf_gpio_pin_set(5);
			LOG_INF("nrf_gpio_pin_set(5)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(5)\n");
			break;						
		case 0x06:
			nrf_gpio_pin_set(6);
			LOG_INF("nrf_gpio_pin_set(6)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(6)\n");
			break;						
		case 0x07:
			nrf_gpio_pin_set(7);
			LOG_INF("nrf_gpio_pin_set(7)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(7)\n");
			break;						
		case 0x08:
			nrf_gpio_pin_set(8);
			LOG_INF("nrf_gpio_pin_set(8)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(8)\n");
			break;						
		case 0x09:
			nrf_gpio_pin_set(9);
			LOG_INF("nrf_gpio_pin_set(9)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(9)\n");
			break;						
		case 0x10:
			nrf_gpio_pin_set(10);
			LOG_INF("nrf_gpio_pin_set(10)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(10)\n");
			break;	
		case 0x11:
			nrf_gpio_pin_set(11);
			LOG_INF("nrf_gpio_pin_set(11)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(11)\n");
			break;
		case 0x12:
			nrf_gpio_pin_set(12);
			LOG_INF("nrf_gpio_pin_set(12)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(12)\n");
			break;		
		case 0x13:
			nrf_gpio_pin_set(13);
			LOG_INF("nrf_gpio_pin_set(13)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(13)\n");
			break;						
		case 0x14:
			nrf_gpio_pin_set(14);
			LOG_INF("nrf_gpio_pin_set(14)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(14)\n");
			break;						
		case 0x15:
			nrf_gpio_pin_set(15);
			LOG_INF("nrf_gpio_pin_set(15)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(15)\n");
			break;						
		case 0x16:
			nrf_gpio_pin_set(16);
			LOG_INF("nrf_gpio_pin_set(16)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(16)\n");
			break;						
		case 0x17:
			nrf_gpio_pin_set(17);
			LOG_INF("nrf_gpio_pin_set(17)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(17)\n");
			break;						
		case 0x18:
			nrf_gpio_pin_set(18);
			LOG_INF("nrf_gpio_pin_set(18)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(18)\n");
			break;						
		case 0x19:
			nrf_gpio_pin_set(19);
			LOG_INF("nrf_gpio_pin_set(19)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(19)\n");
			break;		
		case 0x20:
			nrf_gpio_pin_set(20);
			LOG_INF("nrf_gpio_pin_set(20)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(20)\n");
			break;	
		case 0x21:
			nrf_gpio_pin_set(21);
			LOG_INF("nrf_gpio_pin_set(21)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(21)\n");
			break;
		case 0x22:
			nrf_gpio_pin_set(22);
			LOG_INF("nrf_gpio_pin_set(22)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(22)\n");
			break;		
		case 0x23:
			nrf_gpio_pin_set(23);
			LOG_INF("nrf_gpio_pin_set(23)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(23)\n");
			break;						
		case 0x24:
			nrf_gpio_pin_set(24);
			LOG_INF("nrf_gpio_pin_set(24)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(24)\n");
			break;						
		case 0x25:
			nrf_gpio_pin_set(25);
			LOG_INF("nrf_gpio_pin_set(25)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(25)\n");
			break;						
		case 0x26:
			nrf_gpio_pin_set(26);
			LOG_INF("nrf_gpio_pin_set(26)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(26)\n");
			break;						
		case 0x27:
			nrf_gpio_pin_set(27);
			LOG_INF("nrf_gpio_pin_set(27)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(27)\n");
			break;						
		case 0x28:
			nrf_gpio_pin_set(28);
			LOG_INF("nrf_gpio_pin_set(28)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(28)\n");
			break;						
		case 0x29:
			nrf_gpio_pin_set(29);
			LOG_INF("nrf_gpio_pin_set(29)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(29)\n");
			break;												
		case 0x30:
			nrf_gpio_pin_set(30);
			LOG_INF("nrf_gpio_pin_set(30)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(30)\n");
			break;	
		case 0x31:
			nrf_gpio_pin_set(31);
			LOG_INF("nrf_gpio_pin_set(31)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(31)\n");
			break;		
		case 0x32:
			nrf_gpio_pin_set(32);			//P1.00
			LOG_INF("nrf_gpio_pin_set(32)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(32)\n");
			break;		
		case 0x33:
			nrf_gpio_pin_set(33);			//P1.01
			LOG_INF("nrf_gpio_pin_set(33)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(33)\n");
			break;		
		case 0x34:
			nrf_gpio_pin_set(34);			//P1.02
			LOG_INF("nrf_gpio_pin_set(34)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(34)\n");
			break;		
		case 0x35:
			nrf_gpio_pin_set(35);			//P1.03
			LOG_INF("nrf_gpio_pin_set(35)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(35)\n");
			break;		
		case 0x36:
			nrf_gpio_pin_set(36);			//P1.04
			LOG_INF("nrf_gpio_pin_set(36)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(36)\n");
			break;		
		case 0x37:
			nrf_gpio_pin_set(37);			//P1.05
			LOG_INF("nrf_gpio_pin_set(37)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(37)\n");
			break;		
		case 0x38:
			nrf_gpio_pin_set(38);			//P1.06
			LOG_INF("nrf_gpio_pin_set(38)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(38)\n");
			break;		
		case 0x39:
			nrf_gpio_pin_set(39);			//P1.07
			LOG_INF("nrf_gpio_pin_set(39)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(39)\n");
			break;		
		case 0x40:
			nrf_gpio_pin_set(40);			//P1.08
			LOG_INF("nrf_gpio_pin_set(40)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(40)\n");
			break;		
		case 0x41:
			nrf_gpio_pin_set(41);			//P1.09
			LOG_INF("nrf_gpio_pin_set(41)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(41)\n");
			break;		
		case 0x42:
			nrf_gpio_pin_set(42);			//P1.10
			LOG_INF("nrf_gpio_pin_set(42)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(42)\n");
			break;	
		case 0x43:
			nrf_gpio_pin_set(43);			//P1.11
			LOG_INF("nrf_gpio_pin_set(43)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(43)\n");
			break;	
		case 0x44:
			nrf_gpio_pin_set(44);			//P1.12
			LOG_INF("nrf_gpio_pin_set(44)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(44)\n");
			break;	
		case 0x45:
			nrf_gpio_pin_set(45);			//P1.13
			LOG_INF("nrf_gpio_pin_set(45)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(45)\n");
			break;	
		case 0x46:
			nrf_gpio_pin_set(46);			//P1.14
			LOG_INF("nrf_gpio_pin_set(46)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(46)\n");
			break;	
		case 0x47:
			nrf_gpio_pin_set(47);			//P1.15
			LOG_INF("nrf_gpio_pin_set(47)\n");
			LOG_INF("======================\n");
			printk("nrf_gpio_pin_set(47)\n");
			break;	
		default:

			break;
		}
}
#endif
//////////


static void uart_cb(const struct device *dev, struct uart_event *evt, void *user_data)
{
	ARG_UNUSED(dev);

	static size_t aborted_len;
	struct uart_data_t *buf;
	static uint8_t *aborted_buf;
	static bool disable_req;

	switch (evt->type) {
	case UART_TX_DONE:
		LOG_DBG("UART_TX_DONE");
		if ((evt->data.tx.len == 0) ||
		    (!evt->data.tx.buf)) {
			return;
		}

		if (aborted_buf) {
			buf = CONTAINER_OF(aborted_buf, struct uart_data_t,
					   data);
			aborted_buf = NULL;
			aborted_len = 0;
		} else {
			buf = CONTAINER_OF(evt->data.tx.buf, struct uart_data_t,
					   data);
		}

		k_free(buf);

		buf = k_fifo_get(&fifo_uart_tx_data, K_NO_WAIT);
		if (!buf) {
			return;
		}

		if (uart_tx(uart, buf->data, buf->len, SYS_FOREVER_MS)) {
			LOG_WRN("Failed to send data over UART");
		}

		break;

	case UART_RX_RDY:
		LOG_DBG("UART_RX_RDY");
		buf = CONTAINER_OF(evt->data.rx.buf, struct uart_data_t, data);
		buf->len += evt->data.rx.len;

		if (disable_req) {
			return;
		}

		if ((evt->data.rx.buf[buf->len - 1] == '\n') ||
		    (evt->data.rx.buf[buf->len - 1] == '\r')) {
			disable_req = true;
			uart_rx_disable(uart);
#ifdef Factory_TC
			if(buf->len == 6){
                if(buf->data[0] == 'P' && buf->data[1] == 'I' && buf->data[2] == 'N'  && buf->data[5] == 0x0A){
                    uint8_t which = ((buf->data[3] & 0x0f) << 4) | ((buf->data[4] & 0x0f) );
					printk("UART RX:%c%c%c%c%c\n",buf->data[0],buf->data[1],buf->data[2],buf->data[3],buf->data[4]);
					LOG_INF("UART RX:%c%c%c%c%c\n",buf->data[0],buf->data[1],buf->data[2],buf->data[3],buf->data[4]);
					Factory_Test(which);		//PIN00\n to PIN47\n	
					buf->len = 0;				
                }else{
                    LOG_INF("UART RX length:%d\n",buf->len);
                }
            }else if(buf->len == 16){
                if(buf->data[0] == 'A' && buf->data[1] == 'T' && buf->data[2] == '+'  && buf->data[3] == 'N' && buf->data[4] == 'A' && buf->data[5] == 'M'){
                    name_tH = buf->data[9];
                    name_tL = buf->data[10];
                    name_mH = buf->data[12];
                    name_mL = buf->data[13];
                    command_pending = true;
                }                                    
            }else{
                LOG_INF("UART RX length:%d\n",buf->len);
                //printk("UART RX %c %c %c\n",rx->data[0],rx->data[1],rx->data[2]);
				LOG_HEXDUMP_INF(buf->data, sizeof(buf->data), "TX.log hexdump info");
				//LOG_INF("TX.len = %d\n",buf->len);
            }		
			//buf->len = 0;	
#endif
		}

		break;

	case UART_RX_DISABLED:
		LOG_DBG("UART_RX_DISABLED");
		disable_req = false;

		buf = k_malloc(sizeof(*buf));
		if (buf) {
			buf->len = 0;
		} else {
			LOG_WRN("Not able to allocate UART receive buffer");
			k_work_reschedule(&uart_work, UART_WAIT_FOR_BUF_DELAY);
			return;
		}

		uart_rx_enable(uart, buf->data, sizeof(buf->data),
			       UART_WAIT_FOR_RX);

		break;

	case UART_RX_BUF_REQUEST:
		LOG_DBG("UART_RX_BUF_REQUEST");
		buf = k_malloc(sizeof(*buf));
		if (buf) {
			buf->len = 0;
			uart_rx_buf_rsp(uart, buf->data, sizeof(buf->data));
		} else {
			LOG_WRN("Not able to allocate UART receive buffer");
		}

		break;

	case UART_RX_BUF_RELEASED:
		LOG_DBG("UART_RX_BUF_RELEASED");
		buf = CONTAINER_OF(evt->data.rx_buf.buf, struct uart_data_t,
				   data);

		if (buf->len > 0) {
			k_fifo_put(&fifo_uart_rx_data, buf);
		} else {
			k_free(buf);
		}

		break;

	case UART_TX_ABORTED:
		LOG_DBG("UART_TX_ABORTED");
		if (!aborted_buf) {
			aborted_buf = (uint8_t *)evt->data.tx.buf;
		}

		aborted_len += evt->data.tx.len;
		buf = CONTAINER_OF(aborted_buf, struct uart_data_t,
				   data);

		uart_tx(uart, &buf->data[aborted_len],
			buf->len - aborted_len, SYS_FOREVER_MS);

		break;

	default:
		break;
	}
}

static void uart_work_handler(struct k_work *item)
{
	struct uart_data_t *buf;

	buf = k_malloc(sizeof(*buf));
	if (buf) {
		buf->len = 0;
	} else {
		LOG_WRN("Not able to allocate UART receive buffer");
		k_work_reschedule(&uart_work, UART_WAIT_FOR_BUF_DELAY);
		return;
	}

	uart_rx_enable(uart, buf->data, sizeof(buf->data), UART_WAIT_FOR_RX);
}

static bool uart_test_async_api(const struct device *dev)
{
	const struct uart_driver_api *api =
			(const struct uart_driver_api *)dev->api;

	return (api->callback_set != NULL);
}

static int uart_init(void)
{
	int err;
	int pos;
	struct uart_data_t *rx;
	struct uart_data_t *tx;

	if (!device_is_ready(uart)) {
		return -ENODEV;
	}

	if (IS_ENABLED(CONFIG_USB_DEVICE_STACK)) {
		err = usb_enable(NULL);
		if (err && (err != -EALREADY)) {
			LOG_ERR("Failed to enable USB");
			return err;
		}
	}

	rx = k_malloc(sizeof(*rx));
	if (rx) {
		rx->len = 0;
	} else {
		return -ENOMEM;
	}

	k_work_init_delayable(&uart_work, uart_work_handler);


	if (IS_ENABLED(CONFIG_BT_NUS_UART_ASYNC_ADAPTER) && !uart_test_async_api(uart)) {
		/* Implement API adapter */
		uart_async_adapter_init(async_adapter, uart);
		uart = async_adapter;
	}

	err = uart_callback_set(uart, uart_cb, NULL);
	if (err) {
		k_free(rx);
		LOG_ERR("Cannot initialize UART callback");
		return err;
	}

	if (IS_ENABLED(CONFIG_UART_LINE_CTRL)) {
		LOG_INF("Wait for DTR");
		while (true) {
			uint32_t dtr = 0;

			uart_line_ctrl_get(uart, UART_LINE_CTRL_DTR, &dtr);
			if (dtr) {
				break;
			}
			/* Give CPU resources to low priority threads. */
			k_sleep(K_MSEC(100));
		}
		LOG_INF("DTR set");
		err = uart_line_ctrl_set(uart, UART_LINE_CTRL_DCD, 1);
		if (err) {
			LOG_WRN("Failed to set DCD, ret code %d", err);
		}
		err = uart_line_ctrl_set(uart, UART_LINE_CTRL_DSR, 1);
		if (err) {
			LOG_WRN("Failed to set DSR, ret code %d", err);
		}
	}

	tx = k_malloc(sizeof(*tx));

	if (tx) {
		pos = snprintf(tx->data, sizeof(tx->data),
			       "Starting Nordic UART service example\r\n");

		if ((pos < 0) || (pos >= sizeof(tx->data))) {
			k_free(rx);
			k_free(tx);
			LOG_ERR("snprintf returned %d", pos);
			return -ENOMEM;
		}

		tx->len = pos;
	} else {
		k_free(rx);
		return -ENOMEM;
	}

	err = uart_tx(uart, tx->data, tx->len, SYS_FOREVER_MS);
	if (err) {
		k_free(rx);
		k_free(tx);
		LOG_ERR("Cannot display welcome message (err: %d)", err);
		return err;
	}

	err = uart_rx_enable(uart, rx->data, sizeof(rx->data), 50);
	if (err) {
		LOG_ERR("Cannot enable uart reception (err: %d)", err);
		/* Free the rx buffer only because the tx buffer will be handled in the callback */
		k_free(rx);
	}

	return err;
}

static void connected(struct bt_conn *conn, uint8_t err)
{
	char addr[BT_ADDR_LE_STR_LEN];

	if (err) {
		LOG_ERR("Connection failed (err %u)", err);
		return;
	}

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));
	LOG_INF("Connected MAC address: %s\n", addr);
	printk("Connected MAC address: %s\n", addr);

	current_conn = bt_conn_ref(conn);
	printk("BLE Connected\n");
	LOG_INF("BLE Connected!\n");

	is_connect = true;
#ifdef  nRF21540DK
	nrf_gpio_cfg_output(16);
	nrf_gpio_pin_clear(16);		//Led on
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_set(16);		//Led off
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(16);		//Led on
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_set(16);		//Led off
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(16);		//Led on
	k_sleep(K_MSEC(50));
#elif BT840
	nrf_gpio_cfg_output(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
#elif BT840X
	nrf_gpio_cfg_output(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
#elif BT840XEE
	nrf_gpio_cfg_output(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
#elif BM840
	nrf_gpio_cfg_output(41);	//P1.09
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(41);
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(41);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(41);		//LED_OFF
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(41);		//LED_ON
#elif BM840P
	nrf_gpio_cfg_output(41);	//P1.09
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(41);
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(41);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(41);		//LED_OFF
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(41);		//LED_ON
#elif BT832
	nrf_gpio_cfg_output(20);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(20);
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(20);		
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(20);		//LED_OFF
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(20);		//LED_ON
#elif BT832X
	nrf_gpio_cfg_output(20);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(20);
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(20);		
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(20);		//LED_OFF
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(20);		//LED_ON
#elif BT833
	nrf_gpio_cfg_output(13);
	nrf_gpio_pin_clear(13);		//Led on
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_set(13);		//Led off
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(13);		//Led on
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_set(13);		//Led off
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(13);		//Led on
	k_sleep(K_MSEC(50));
#elif BT840N_V4	
	nrf_gpio_cfg_output(13);
	nrf_gpio_pin_clear(13);		//Led on
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_set(13);		//Led off
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(13);		//Led on
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_set(13);		//Led off
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(13);		//Led on
	k_sleep(K_MSEC(50));
#elif BT40
	nrf_gpio_cfg_output(31); 
	nrf_gpio_pin_clear(31);	//Led on
	k_sleep(K_MSEC(50));	
	nrf_gpio_pin_set(31);	//Led off		
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(31);	//Led on
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_set(31);	//Led off	
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(31);	//Led on
	k_sleep(K_MSEC(50));	
#elif BT40N_V6
	nrf_gpio_cfg_output(31); 
	nrf_gpio_pin_clear(31);	//Led on
	k_sleep(K_MSEC(50));	
	nrf_gpio_pin_set(31);	//Led off		
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(31);	//Led on
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_set(31);	//Led off	
	k_sleep(K_MSEC(50));
	nrf_gpio_pin_clear(31);	//Led on
	k_sleep(K_MSEC(50));	
#endif

	//dk_set_led_on(CON_STATUS_LED);
}

static void disconnected(struct bt_conn *conn, uint8_t reason)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	LOG_INF("Disconnected MAC address: %s (reason %u)", addr, reason);
	printk("Disconnected MAC address: %s (reason %u)", addr, reason);
	printk("BLE Disconnected!\n");
	if (auth_conn) {
		bt_conn_unref(auth_conn);
		auth_conn = NULL;
	}

	if (current_conn) {
		bt_conn_unref(current_conn);
		current_conn = NULL;
		//dk_set_led_off(CON_STATUS_LED);		//LED2_OFF
	}


#ifdef nRF21540DK
	nrf_gpio_cfg_output(16);
	nrf_gpio_pin_set(16);		//Led off
#elif BT840
	nrf_gpio_cfg_output(13);
	nrf_gpio_pin_set(13);		//Led off
#elif BT840X
	nrf_gpio_cfg_output(13);
	nrf_gpio_pin_set(13);		//Led off
#elif BT840XEE
	nrf_gpio_cfg_output(13);
	nrf_gpio_pin_set(13);		//Led off
#elif BM840
	nrf_gpio_cfg_output(41);
	nrf_gpio_pin_set(41);		//Led off
#elif BM840P
	nrf_gpio_cfg_output(41);
	nrf_gpio_pin_set(41);		//Led off
#elif BT832
	nrf_gpio_cfg_output(20);
	nrf_gpio_pin_set(20);		//Led off
#elif BT832X
	nrf_gpio_cfg_output(20);
	nrf_gpio_pin_set(20);		//Led off
#elif BT833
	nrf_gpio_cfg_output(13);
	nrf_gpio_pin_set(13);		//Led off
#elif BT840N_V4	
	nrf_gpio_cfg_output(13);
	nrf_gpio_pin_set(13);		//Led off
#elif BT40
	nrf_gpio_cfg_output(31); 
	nrf_gpio_pin_set(31);	//Led off	
#elif BT40N_V6
	nrf_gpio_cfg_output(31); 
	nrf_gpio_pin_set(31);	//Led off	
#endif

}

#ifdef CONFIG_BT_NUS_SECURITY_ENABLED
static void security_changed(struct bt_conn *conn, bt_security_t level,
			     enum bt_security_err err)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	if (!err) {
		LOG_INF("Security changed: %s level %u", addr, level);
	} else {
		LOG_WRN("Security failed: %s level %u err %d", addr,
			level, err);
	}
}
#endif

BT_CONN_CB_DEFINE(conn_callbacks) = {
	.connected    = connected,
	.disconnected = disconnected,
#ifdef CONFIG_BT_NUS_SECURITY_ENABLED
	.security_changed = security_changed,
#endif
};

#if defined(CONFIG_BT_NUS_SECURITY_ENABLED)
static void auth_passkey_display(struct bt_conn *conn, unsigned int passkey)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	LOG_INF("Passkey for %s: %06u", addr, passkey);
}

static void auth_passkey_confirm(struct bt_conn *conn, unsigned int passkey)
{
	char addr[BT_ADDR_LE_STR_LEN];

	auth_conn = bt_conn_ref(conn);

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	LOG_INF("Passkey for %s: %06u", addr, passkey);
	LOG_INF("Press Button 1 to confirm, Button 2 to reject.");
}


static void auth_cancel(struct bt_conn *conn)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	LOG_INF("Pairing cancelled: %s", addr);
}


static void pairing_complete(struct bt_conn *conn, bool bonded)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	LOG_INF("Pairing completed: %s, bonded: %d", addr, bonded);
}


static void pairing_failed(struct bt_conn *conn, enum bt_security_err reason)
{
	char addr[BT_ADDR_LE_STR_LEN];

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, sizeof(addr));

	LOG_INF("Pairing failed conn: %s, reason %d", addr, reason);
}


static struct bt_conn_auth_cb conn_auth_callbacks = {
	.passkey_display = auth_passkey_display,
	.passkey_confirm = auth_passkey_confirm,
	.cancel = auth_cancel,
};

static struct bt_conn_auth_info_cb conn_auth_info_callbacks = {
	.pairing_complete = pairing_complete,
	.pairing_failed = pairing_failed
};
#else
static struct bt_conn_auth_cb conn_auth_callbacks;
//static struct bt_conn_auth_info_cb conn_auth_info_callbacks;
#endif

static void bt_receive_cb(struct bt_conn *conn, const uint8_t *const data,
			  uint16_t len)
{
	int err;
	char addr[BT_ADDR_LE_STR_LEN] = {0};

	bt_addr_le_to_str(bt_conn_get_dst(conn), addr, ARRAY_SIZE(addr));

	LOG_INF("Received data from: %s", addr);

	for (uint16_t pos = 0; pos != len;) {
		struct uart_data_t *tx = k_malloc(sizeof(*tx));

		if (!tx) {
			LOG_WRN("Not able to allocate UART send data buffer");
			return;
		}

		/* Keep the last byte of TX buffer for potential LF char. */
		size_t tx_data_size = sizeof(tx->data) - 1;

		if ((len - pos) > tx_data_size) {
			tx->len = tx_data_size;
		} else {
			tx->len = (len - pos);
		}

		memcpy(tx->data, &data[pos], tx->len);

		pos += tx->len;

		/* Append the LF character when the CR character triggered
		 * transmission from the peer.
		 */
		if ((pos == len) && (data[len - 1] == '\r')) {
			tx->data[tx->len] = '\n';
			tx->len++;
		}

		err = uart_tx(uart, tx->data, tx->len, SYS_FOREVER_MS);
		if (err) {
			k_fifo_put(&fifo_uart_tx_data, tx);
		}
/////		
		//LOG_HEXDUMP_INF(tx->data, sizeof(tx->data), "RX.log hexdump info");
		//LOG_INF("RX.len = %d\n",tx->len);
		printk("RX.len = %d\n",tx->len);

		if(tx->len == 6){
        	if(tx->data[0] == 'L' && tx->data[1] == 'e' && tx->data[2] == 'd'  && tx->data[3] == 'O'){
				if(tx->data[4] == 'n'){
			#ifdef BT840N_V4
				nrf_gpio_cfg_output(17); 
				nrf_gpio_cfg_output(32);	//P1.00 
				nrf_gpio_pin_clear(17);		//Led on
				nrf_gpio_pin_clear(32);		//Led on
			#elif BT840
				nrf_gpio_cfg_output(32);	//P1.00 
				nrf_gpio_pin_clear(32);		//Led on
			#elif BT840X
				nrf_gpio_cfg_output(32);	//P1.00 
				nrf_gpio_pin_clear(32);		//Led on
			#elif BT840XEE
				nrf_gpio_cfg_output(32);	//P1.00 
				nrf_gpio_pin_clear(32);		//Led on
			#elif BM840
				nrf_gpio_cfg_output(40);	//P1.08 
				nrf_gpio_pin_clear(40);		//Led on
			#elif BM840P
				nrf_gpio_cfg_output(40);	//P1.08 
				nrf_gpio_pin_clear(40);		//Led on
			#elif BT832
				nrf_gpio_cfg_output(18);	
				nrf_gpio_pin_clear(18);		//Led on
			#elif BT832X
				nrf_gpio_cfg_output(18);	
				nrf_gpio_pin_clear(18);		//Led on
			#elif BT833
				nrf_gpio_cfg_output(32);		
				nrf_gpio_pin_clear(32);		//Led on
			#elif BT40
				nrf_gpio_cfg_output(28); 
				nrf_gpio_cfg_output(29); 
				nrf_gpio_cfg_output(30);	 
				nrf_gpio_pin_clear(28);		//Led on
				nrf_gpio_pin_clear(29);		//Led on
				nrf_gpio_pin_clear(30);		//Led on
			#elif BT40N_V6
				nrf_gpio_cfg_output(28); 
				nrf_gpio_cfg_output(29); 
				nrf_gpio_cfg_output(30);	 
				nrf_gpio_pin_clear(28);		//Led on
				nrf_gpio_pin_clear(29);		//Led on
				nrf_gpio_pin_clear(30);		//Led on
			#endif
				}else if(tx->data[4] == 'f'){
			#ifdef BT840N_V4
				nrf_gpio_cfg_output(17); 
				nrf_gpio_cfg_output(32);	//P1.00 
				nrf_gpio_pin_set(17);		//Led off
				nrf_gpio_pin_set(32);		//Led off	
			#elif BT840
				nrf_gpio_cfg_output(32);	//P1.00 
				nrf_gpio_pin_set(32);		//Led off
			#elif BT840X
				nrf_gpio_cfg_output(32);	//P1.00 
				nrf_gpio_pin_set(32);		//Led off
			#elif BT840XEE
				nrf_gpio_cfg_output(32);	//P1.00 
				nrf_gpio_pin_set(32);		//Led off
			#elif BM840
				nrf_gpio_cfg_output(40);		//P1.08 
				nrf_gpio_pin_set(40);		//Led off
			#elif BM840P
				nrf_gpio_cfg_output(40);		//P1.08 
				nrf_gpio_pin_set(40);		//Led off
			#elif BT832
				nrf_gpio_cfg_output(18);	
				nrf_gpio_pin_set(18);		//Led off
			#elif BT832X
				nrf_gpio_cfg_output(18);	
				nrf_gpio_pin_set(18);		//Led off
			#elif BT833
				nrf_gpio_cfg_output(32); 
				nrf_gpio_pin_set(32);		//Led off
			#elif BT40		
				nrf_gpio_cfg_output(28); 
				nrf_gpio_cfg_output(29); 
				nrf_gpio_cfg_output(30);	
				nrf_gpio_pin_set(28);		//Led off
				nrf_gpio_pin_set(29);		//Led off
				nrf_gpio_pin_set(30);		//Led off
			#elif BT40N_V6			
				nrf_gpio_cfg_output(28); 
				nrf_gpio_cfg_output(29); 
				nrf_gpio_cfg_output(30);	
				nrf_gpio_pin_set(28);		//Led off
				nrf_gpio_pin_set(29);		//Led off
				nrf_gpio_pin_set(30);		//Led off
			#endif
				}
					
			}                                                 
    	}
/////
	}
}

static struct bt_nus_cb nus_cb = {
	.received = bt_receive_cb,
};

void error(void)
{
	//dk_set_leds_state(DK_ALL_LEDS_MSK, DK_NO_LEDS_MSK);

	while (true) {
		/* Spin for ever */
		k_sleep(K_MSEC(1000));
	}
}

#ifdef CONFIG_BT_NUS_SECURITY_ENABLED
static void num_comp_reply(bool accept)
{
	if (accept) {
		bt_conn_auth_passkey_confirm(auth_conn);
		LOG_INF("Numeric Match, conn %p", (void *)auth_conn);
	} else {
		bt_conn_auth_cancel(auth_conn);
		LOG_INF("Numeric Reject, conn %p", (void *)auth_conn);
	}

	bt_conn_unref(auth_conn);
	auth_conn = NULL;
}

/////
static struct button_handler button = {
	.cb = button_changed,
};
/////

void button_changed(uint32_t button_state, uint32_t has_changed)
{
	uint32_t buttons = button_state & has_changed;

	if (auth_conn) {
		if (buttons & KEY_PASSKEY_ACCEPT) {
			num_comp_reply(true);
		}

		if (buttons & KEY_PASSKEY_REJECT) {
			num_comp_reply(false);
		}
	}
/////
	if (buttons & DK_BTN1_MSK) {
		bnt_flag = true;
		bnt1_val=0x3E;
		printk("Button 1 pressed!\n");		
	} else if (buttons & DK_BTN2_MSK) {
		bnt_flag = true;
		bnt2_val=0x3F;
		printk("Button 2 pressed!\n");
	} else if (buttons & DK_BTN3_MSK) {
		bnt_flag = true;
		bnt3_val=0x30;
		printk("Button 3 pressed!\n");
	} else if (buttons & DK_BTN4_MSK) {
		bnt_flag = true;
		bnt4_val=0x31;
		printk("Button 4 pressed!\n");
	}

	if(bnt_flag == true)
		{
			bnt_flag = false;
			uint8_t send_str[5];
			if(bnt1_val == 0x3E){
				send_str[0] = 'B';
				send_str[1] = 'u';
				send_str[2] = 't';
				send_str[3] = bnt1_val;
				send_str[4] = '\n';
				bt_nus_send(NULL, send_str, 5);
				bnt1_val = 0x00;
			}else if(bnt2_val == 0x3F){
				send_str[0] = 'B';
				send_str[1] = 'u';
				send_str[2] = 't';
				send_str[3] = bnt2_val;
				send_str[4] = '\n';
				bt_nus_send(NULL, send_str, 5);
				bnt2_val = 0x00;
			}else if(bnt3_val == 0x30){
				send_str[0] = 'B';
				send_str[1] = 'u';
				send_str[2] = 't';
				send_str[3] = bnt3_val;
				send_str[4] = '\n';
				bt_nus_send(NULL, send_str, 5);
				bnt3_val = 0x00;
			}else if(bnt4_val == 0x31){
				send_str[0] = 'B';
				send_str[1] = 'u';
				send_str[2] = 't';
				send_str[3] = bnt4_val;
				send_str[4] = '\n';
				bt_nus_send(NULL, send_str, 5);
				bnt4_val = 0x00;
			}
		}
/////
}
#endif /* CONFIG_BT_NUS_SECURITY_ENABLED */

static void configure_gpio(void)
{
	int err;

#ifdef CONFIG_BT_NUS_SECURITY_ENABLED
/////
	//err = dk_buttons_init(button_changed);
	err = dk_buttons_init(NULL);
/////
	if (err) {
		LOG_ERR("Cannot init buttons (err: %d)", err);
	}
	
/////
	dk_button_handler_add(&button);
/////
#endif /* CONFIG_BT_NUS_SECURITY_ENABLED */

/*
	err = dk_leds_init();
	if (err) {
		LOG_ERR("Cannot init LEDs (err: %d)", err);
	}
	*/

#ifdef nRF21540DK
	nrf_gpio_cfg_output(13); 
	nrf_gpio_cfg_output(14); 
	nrf_gpio_cfg_output(15); 
	nrf_gpio_cfg_output(16);
#elif BT840
	nrf_gpio_cfg_output(13);
	nrf_gpio_cfg_output(32);	//P1.00	
#elif BT840X
	nrf_gpio_cfg_output(13);
	nrf_gpio_cfg_output(32);	//P1.00	
#elif BT840XEE
	#define APP_ANTSEL_PIN	15	
	nrf_gpio_cfg_input(3,GPIO_PULL_UP);
	nrf_gpio_cfg_output(13);
	nrf_gpio_cfg_output(32);	//P1.00	
	nrf_gpio_cfg_output(APP_ANTSEL_PIN); 
#elif BM840
	nrf_gpio_cfg_output(40);	//P1.08	
	nrf_gpio_cfg_output(41);	//P1.09	
#elif BM840P
	nrf_gpio_cfg_output(40);	//P1.08	
	nrf_gpio_cfg_output(41);	//P1.09	
#elif BT832
	nrf_gpio_cfg_output(18);
	nrf_gpio_cfg_output(20);
#elif BT832X
	nrf_gpio_cfg_output(18);
	nrf_gpio_cfg_output(20);
#elif BT833
	nrf_gpio_cfg_output(32);
	nrf_gpio_cfg_output(13);
#elif BT840N_V4	
	#define APP_ANTSEL_PIN	20
	nrf_gpio_cfg_input(3,GPIO_PULL_UP);
	nrf_gpio_cfg_output(13); 
	nrf_gpio_cfg_output(17); 
	nrf_gpio_cfg_output(19); 
	nrf_gpio_cfg_output(32);	//P1.00 
	nrf_gpio_cfg_output(APP_ANTSEL_PIN); 
#elif BT40
	nrf_gpio_cfg_output(28); 
	nrf_gpio_cfg_output(29); 
	nrf_gpio_cfg_output(30); 
	nrf_gpio_cfg_output(31); 			
#elif BT40N_V6
	#define APP_ANTSEL_PIN	37		//P1.05	
	nrf_gpio_cfg_input(5,GPIO_PULL_UP);
	nrf_gpio_cfg_output(28); 
	nrf_gpio_cfg_output(29); 
	nrf_gpio_cfg_output(30); 
	nrf_gpio_cfg_output(31); 
	nrf_gpio_cfg_output(APP_ANTSEL_PIN); 	//P1.05	
#endif

#ifdef nRF21540DK
	nrf_gpio_pin_clear(13);		//Led on
	nrf_gpio_pin_clear(14);		//Led on
	nrf_gpio_pin_clear(15);		//Led on
	nrf_gpio_pin_clear(16);		//Led on
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);		//Led off
	nrf_gpio_pin_set(14);		//Led off
	nrf_gpio_pin_set(15);		//Led off
	nrf_gpio_pin_set(16);		//Led off
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);		//Led on
	nrf_gpio_pin_clear(14);		//Led on
	nrf_gpio_pin_clear(15);		//Led on
	nrf_gpio_pin_clear(16);		//Led on
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);		//Led off
	nrf_gpio_pin_set(14);		//Led off
	nrf_gpio_pin_set(15);		//Led off
	nrf_gpio_pin_set(16);		//Led off
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);		//Led on
	nrf_gpio_pin_clear(14);		//Led on
	nrf_gpio_pin_clear(15);		//Led on
	nrf_gpio_pin_clear(16);		//Led on
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);		//Led off
	nrf_gpio_pin_set(14);		//Led off
	nrf_gpio_pin_set(15);		//Led off
	nrf_gpio_pin_set(16);		//Led off
#elif BT840
	nrf_gpio_pin_clear(13);
	nrf_gpio_pin_clear(32);	//P1.00
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	nrf_gpio_pin_set(32);	//P1.00
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
	nrf_gpio_pin_clear(32);	//P1.00
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	nrf_gpio_pin_set(32);	//P1.00
#elif BT840X
	nrf_gpio_pin_clear(13);
	nrf_gpio_pin_clear(32);	//P1.00
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	nrf_gpio_pin_set(32);	//P1.00
    k_sleep(K_MSEC(300));
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
	nrf_gpio_pin_clear(32);	//P1.00
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	nrf_gpio_pin_set(32);	//P1.00
#elif BT840XEE
	nrf_gpio_pin_clear(13);
	nrf_gpio_pin_clear(32);	//P1.00
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	nrf_gpio_pin_set(32);	//P1.00
    k_sleep(K_MSEC(300));
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);
	nrf_gpio_pin_clear(32);	//P1.00
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);
	nrf_gpio_pin_set(32);	//P1.00
	//uint8_t rc = gpio_pin_get(led_dev,3);
	uint8_t rc = nrf_gpio_pin_read(3);
	if(rc == 0){		
        //gpio_pin_set(led_dev, APP_ANTSEL_PIN, 1);
		nrf_gpio_pin_clear(APP_ANTSEL_PIN);
		printk("ANTSEL_PIN(P0.15) low \n");
    }else{
        //gpio_pin_set(led_dev, APP_ANTSEL_PIN, 0);
		nrf_gpio_pin_set(APP_ANTSEL_PIN);
		printk("ANTSEL_PIN(P0.15) high \n");
    } 
#elif BM840
	nrf_gpio_pin_clear(40);	//P1.08
	nrf_gpio_pin_clear(41);	//P1.090
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(40);
	nrf_gpio_pin_set(41);	
    k_sleep(K_MSEC(300));
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(40);
	nrf_gpio_pin_clear(41);	
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(40);
	nrf_gpio_pin_set(41);	
#elif BM840P
	nrf_gpio_pin_clear(40);	//P1.08
	nrf_gpio_pin_clear(41);	//P1.090
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(40);
	nrf_gpio_pin_set(41);	
    k_sleep(K_MSEC(300));
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(40);
	nrf_gpio_pin_clear(41);	
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(40);
	nrf_gpio_pin_set(41);
#elif BT832
	nrf_gpio_pin_clear(18);
	nrf_gpio_pin_clear(20);
	k_sleep(K_MSEC(300));	
	nrf_gpio_pin_set(18);
	nrf_gpio_pin_set(20);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(18);
	nrf_gpio_pin_clear(20);
	k_sleep(K_MSEC(300));	
	nrf_gpio_pin_set(18);
	nrf_gpio_pin_set(20);
	k_sleep(K_MSEC(300));
#elif BT832X
	nrf_gpio_pin_clear(18);
	nrf_gpio_pin_clear(20);
	k_sleep(K_MSEC(300));	
	nrf_gpio_pin_set(18);
	nrf_gpio_pin_set(20);
	k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(18);
	nrf_gpio_pin_clear(20);
	k_sleep(K_MSEC(300));	
	nrf_gpio_pin_set(18);
	nrf_gpio_pin_set(20);
	k_sleep(K_MSEC(300));
#elif BT833
	nrf_gpio_pin_clear(13);		//Led on
	nrf_gpio_pin_clear(32);		//Led on,P1.00
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);		//Led off
	nrf_gpio_pin_set(32);		//Led off,P1.00
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);		//Led on
	nrf_gpio_pin_clear(32);		//Led on,P1.00
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);		//Led off
	nrf_gpio_pin_set(32);		//Led off,P1.00
#elif BT840N_V4	
	nrf_gpio_pin_clear(13);		//Led on
	nrf_gpio_pin_clear(17);		//Led on
	nrf_gpio_pin_clear(19);		//Led on
	nrf_gpio_pin_clear(32);		//Led on,P1.00
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);		//Led off
	nrf_gpio_pin_set(17);		//Led off
	nrf_gpio_pin_set(19);		//Led off
	nrf_gpio_pin_set(32);		//Led off,P1.00
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(13);		//Led on
	nrf_gpio_pin_clear(17);		//Led on
	nrf_gpio_pin_clear(19);		//Led on
	nrf_gpio_pin_clear(32);		//Led on,P1.00
        k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(13);		//Led off
	nrf_gpio_pin_set(17);		//Led off
	nrf_gpio_pin_set(19);		//Led off
	nrf_gpio_pin_set(32);		//Led off,P1.00
        k_sleep(K_MSEC(300));
	//uint8_t rc = gpio_pin_get(led_dev,3);
	uint8_t rc = nrf_gpio_pin_read(3);
	if(rc == 0){		
        //gpio_pin_set(led_dev, APP_ANTSEL_PIN, 1);
		nrf_gpio_pin_clear(APP_ANTSEL_PIN);
		printk("ANTSEL_PIN(P0.20) low \n");
    }else{
        //gpio_pin_set(led_dev, APP_ANTSEL_PIN, 0);
		nrf_gpio_pin_set(APP_ANTSEL_PIN);
		printk("ANTSEL_PIN(P0.20) high \n");
    } 
#elif BT40
	nrf_gpio_pin_clear(28);		//Led on
	nrf_gpio_pin_clear(29);		//Led on
	nrf_gpio_pin_clear(30);		//Led on
	nrf_gpio_pin_clear(31);		//Led on
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(28);		//Led off
	nrf_gpio_pin_set(29);		//Led off
	nrf_gpio_pin_set(30);		//Led off
	nrf_gpio_pin_set(31);		//Led off
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(28);		//Led on
	nrf_gpio_pin_clear(29);		//Led on
	nrf_gpio_pin_clear(30);		//Led on
	nrf_gpio_pin_clear(31);		//Led on
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(28);		//Led off
	nrf_gpio_pin_set(29);		//Led off
	nrf_gpio_pin_set(30);		//Led off
	nrf_gpio_pin_set(31);		//Led off
    k_sleep(K_MSEC(300));
#elif BT40N_V6
	nrf_gpio_pin_clear(28);		//Led on
	nrf_gpio_pin_clear(29);		//Led on
	nrf_gpio_pin_clear(30);		//Led on
	nrf_gpio_pin_clear(31);		//Led on
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(28);		//Led off
	nrf_gpio_pin_set(29);		//Led off
	nrf_gpio_pin_set(30);		//Led off
	nrf_gpio_pin_set(31);		//Led off
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_clear(28);		//Led on
	nrf_gpio_pin_clear(29);		//Led on
	nrf_gpio_pin_clear(30);		//Led on
	nrf_gpio_pin_clear(31);		//Led on
    k_sleep(K_MSEC(300));
	nrf_gpio_pin_set(28);		//Led off
	nrf_gpio_pin_set(29);		//Led off
	nrf_gpio_pin_set(30);		//Led off
	nrf_gpio_pin_set(31);		//Led off
    k_sleep(K_MSEC(300));
    //uint8_t rc2 = gpio_pin_get(led_dev,5);
	uint8_t rc2 = nrf_gpio_pin_read(5);
	if(rc2 == 0){		
        //gpio_pin_set(led_dev1, 05, 1);
		nrf_gpio_pin_clear(APP_ANTSEL_PIN);	
		printk("ANT_SEL(P1.05) low \n");
    }else{
        //gpio_pin_set(led_dev1, 05, 0);		
		nrf_gpio_pin_set(APP_ANTSEL_PIN);
		printk("ANT_SEL(P1.05) high \n");
    }  		
#endif	
}

#ifdef BT40N_V6
	#ifdef DCDC_VOLTAGE
int change_gpio_voltage(uint32_t target_voltage)
{
    int err = 0;
    #if defined(CONFIG_BOARD_NRF5340DK_NRF5340_CPUAPP)
    
    uint32_t regout = NRF_UICR->VREGHVOUT;
    LOG_INF("REGOUT = 0x%08x", regout);
    
    if ((regout & UICR_VREGHVOUT_VREGHVOUT_Msk)  != target_voltage) {
        LOG_INF("Target voltage not set. Configuring");

        // Set NVMC in write mode:
        NRF_NVMC->CONFIG = NVMC_CONFIG_WEN_Wen << NVMC_CONFIG_WEN_Pos;
        while (NRF_NVMC->CONFIG != NVMC_CONFIG_WEN_Wen << NVMC_CONFIG_WEN_Pos) {
            // Wait...
        }

        // Write the actual UICR Register:
        NRF_UICR->VREGHVOUT = (target_voltage | ~UICR_VREGHVOUT_VREGHVOUT_Msk);
        while (NRF_NVMC->READY == NVMC_READY_READY_Busy) {
            // Wait...
        }

        // Set NVMC back in read mode:
        NRF_NVMC->CONFIG = NVMC_CONFIG_WEN_Ren << NVMC_CONFIG_WEN_Pos;
        while (NRF_NVMC->CONFIG != NVMC_CONFIG_WEN_Ren << NVMC_CONFIG_WEN_Pos) {
            // Wait...
        }
        // Check whether it was set successfully:
        if ((NRF_UICR->VREGHVOUT & UICR_VREGHVOUT_VREGHVOUT_Msk) != target_voltage) {
            err = 0;
        }
        // Reset if success. Config will remain on future reboots.
        if (err == 0) {
            NVIC_SystemReset();
        }
    }

    return err;
    #else
    // implement for other chips
    return err;
    #endif
}
	#endif
#endif

#ifdef BLE_ADV_NAME
static void Command_Pro(){
        int err = bt_le_adv_stop();
        if (err) {
           LOG_INF("Advertising failed to stop (err %d)\n", err);
        }else{
           printk("OK\n");
        }
	/*	
		char temp_name[] = {'M','9','9',' ','9','9'};
        temp_name[1] = name_tH;
        temp_name[2] = name_tL;
        temp_name[4] = name_mH;
        temp_name[5] = name_mL;

		struct bt_data adname[] = {
		BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
		BT_DATA(BT_DATA_NAME_COMPLETE, temp_name, 6),
		};	
	*/	
		char temp_name[] = {'E','V','-','M','9','9',' ','9','9'};
        temp_name[4] = name_tH;
        temp_name[5] = name_tL;
        temp_name[7] = name_mH;
        temp_name[8] = name_mL;

		struct bt_data adname[] = {
		BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
		BT_DATA(BT_DATA_NAME_COMPLETE, temp_name, 9),
		};	
		
		err = bt_le_adv_start(BT_LE_ADV_CONN, adname, ARRAY_SIZE(adname), sd, ARRAY_SIZE(sd));

        if (err) {
           printk("Advertising failed to start (err %d)\n", err);
        }else{
           printk("Advertising restart \n");
        }				  
/*		
        err = bt_set_name(temp_name);
        if (err) {
           LOG_INF("BLE set name error (err %d)\n", err);
        }else{
           LOG_INF("BLE set name success\n");
        }
        err = bt_le_adv_start(BT_LE_ADV_CONN_NAME, ad, ARRAY_SIZE(ad), NULL, 0);
        if (err) {
           LOG_INF("Advertising failed to start (err %d)\n", err);
        }else{
           LOG_INF("Advertising restart \n");
        }
*/
		
}

#endif

#ifdef BT40
#elif BT40N_V6
#elif BT832
#else
	#ifdef MONITOR_TXPOWER_SETTING
void SWI3_IRQHandler(void)
{
    /* Clear event register */
    NRF_EGU3->EVENTS_TRIGGERED[0] = 0;
    LOG_INF("TX power: %d dBm", (int8_t)NRF_RADIO->TXPOWER);
}

void monitor_tx_power_register(void)
{
    IRQ_CONNECT(SWI3_IRQn, 5, SWI3_IRQHandler, NULL, 0);
    NVIC_EnableIRQ(SWI3_IRQn);
    NRF_EGU3->INTENSET = EGU_INTEN_TRIGGERED0_Msk;
    NRF_PPI->CH[0].EEP = (uint32_t) &NRF_RADIO->EVENTS_TXREADY;
    NRF_PPI->CH[0].TEP = (uint32_t) &NRF_EGU3->TASKS_TRIGGER[0];
    NRF_PPI->CHENSET = PPI_CHENSET_CH0_Msk;    
}
	#endif
#endif

static int init_button(void)
{
	int err;

	err = dk_buttons_init(button_changed);
	if (err) {
		//LOG_ERR("Cannot init LEDs (err: %d)", err);
		printk("Cannot init buttons (err: %d)\n", err);
	}

	//return err;
}

int main(void)
{
	//int blink_status = 0;
	int err = 0;

	configure_gpio();
	
	//led_dev = DEVICE_DT_GET(DT_NODELABEL(gpio0));
	//led_dev1 = DEVICE_DT_GET(DT_NODELABEL(gpio1));

	err = uart_init();
	if (err) {
		error();
	}

	if (IS_ENABLED(CONFIG_BT_NUS_SECURITY_ENABLED)) {
		err = bt_conn_auth_cb_register(&conn_auth_callbacks);
		if (err) {
			printk("Failed to register authorization callbacks.\n");
			LOG_INF("Failed to register authorization callbacks.\n");
			return 0;
		}

		err = bt_conn_auth_info_cb_register(&conn_auth_info_callbacks);
		if (err) {
			printk("Failed to register authorization info callbacks.\n");
			LOG_INF("Failed to register authorization info callbacks.\n");
			return 0;
		}
	}
	
#ifdef BT40
#elif BT832
#elif BT40N_V6
#else
	#ifdef MONITOR_TXPOWER_SETTING
    monitor_tx_power_register();
	#endif
#endif

	err = bt_enable(NULL);
	if (err) {
		error();
	}

	LOG_INF("Bluetooth initialized");

	k_sem_give(&ble_init_ok);

	if (IS_ENABLED(CONFIG_SETTINGS)) {
		settings_load();
	}	
	
	err = bt_nus_init(&nus_cb);
	if (err) {
		LOG_ERR("Failed to initialize UART service (err: %d)", err);
		return;
	}
	
	err = bt_le_adv_stop();
	if (err) {
        LOG_INF("Advertising failed to stop (err %d)\n", err);
    }else{
        LOG_INF("Advertising stop \n");
    }

#ifdef nRF21540DK
	printk("Set ANTSEL_PIN(20) high \n");
	err = bt_set_name("21540_ANT_H");
	if (err) {
        printk("Set Name error:(err %d)\n", err);
    }
#elif BT840
    //err = bt_set_name("BT840");
	char temp_name[] = {'E','V','-','B','T','8','4','0',' ',' '};
	if (err) {
        printk("BLE set name error (err %d)\n", err);
    }else{
        printk("BLE set name success\n");
    }
#elif BT840X
	nrf_gpio_cfg_output(APP_CHL_PIN);
    nrf_gpio_cfg_output(APP_CPS_PIN);
	nrf_gpio_pin_set( APP_CHL_PIN);
	nrf_gpio_pin_clear( APP_CPS_PIN);
	#ifdef MONITOR_TXPOWER_SETTING
    	monitor_tx_power_register();
	#endif

    //err = bt_set_name("BT840X");
	char temp_name[] = {'E','V','-','B','T','8','4','0','X',' '};
	if (err) {
        printk("BLE set name error (err %d)\n", err);
    }else{
        printk("BLE set name success\n");
    }
#elif BT840XEE
	nrf_gpio_cfg_output(APP_CHL_PIN);
    nrf_gpio_cfg_output(APP_CPS_PIN);
	nrf_gpio_pin_set( APP_CHL_PIN);
	nrf_gpio_pin_clear( APP_CPS_PIN);
	//uint8_t rc3 = gpio_pin_get(led_dev,03);
	uint8_t rc3 = nrf_gpio_pin_read(3);
	char temp_name[] = {'E','V','-','8','4','0','X','E','E','1'};
	if(rc3 == 0){		
        //gpio_pin_set(led_dev, APP_ANTSEL_PIN, 1);
		nrf_gpio_pin_clear(APP_ANTSEL_PIN);		
		printk("Set ANTSEL_PIN(P0.15) low \n");
		//err = bt_set_name("BT840XEE ANT1");
		
    }else{
        //gpio_pin_set(led_dev, APP_ANTSEL_PIN, 0);
		nrf_gpio_pin_set(APP_ANTSEL_PIN);
		printk("Set ANTSEL_PIN(P0.15) high \n");
		//err = bt_set_name("BT840XEE ANT2");
		temp_name[9] = '2';
    }  	
    if (err) {
        printk("Set Name error:(err %d)\n", err);
    }
#elif BM840
    //err = bt_set_name("BM840");
	char temp_name[] = {'E','V','-','B','M','8','4','0',' ',' '};
	if (err) {
        printk("BLE set name error (err %d)\n", err);
    }else{
        printk("BLE set name success\n");
    }
#elif BM840P
    //err = bt_set_name("BM840");
	char temp_name[] = {'E','V','-','B','M','8','4','0','P',' '};
	if (err) {
        printk("BLE set name error (err %d)\n", err);
    }else{
        printk("BLE set name success\n");
    }
#elif BT832
    //err = bt_set_name("BT832");
	char temp_name[] = {'E','V','-','B','T','8','3','2',' ',' '};
	if (err) {
        printk("BLE set name error (err %d)\n", err);
    }else{
        printk("BLE set name success\n");
    }
#elif BT832X
    nrf_gpio_cfg_output(APP_CPS_PIN);
	nrf_gpio_pin_clear( APP_CPS_PIN);
	#ifdef MONITOR_TXPOWER_SETTING
    	monitor_tx_power_register();
	#endif

	char temp_name[] = {'E','V','-','B','T','8','3','2','X',' '};
	if (err) {
        printk("BLE set name error (err %d)\n", err);
    }else{
        printk("BLE set name success\n");
    }
#elif BT833
    //err = bt_set_name("BT833");
	char temp_name[] = {'E','V','-','B','T','8','3','3',' ',' '};
	if (err) {
        printk("BLE set name error (err %d)\n", err);
    }else{
        printk("BLE set name success\n");
    }
#elif BT840N_V4	
	//uint8_t rc3 = gpio_pin_get(led_dev,03);
	uint8_t rc3 = nrf_gpio_pin_read(3);
	char temp_name[] = {'E','V','-','B','T','8','4','0','N','1'};
	if(rc3 == 0){		
        //gpio_pin_set(led_dev, APP_ANTSEL_PIN, 1);
		nrf_gpio_pin_clear(APP_ANTSEL_PIN);		
		printk("Set ANTSEL_PIN(20) low \n");
		//err = bt_set_name("BT840N ANT1");
		
    }else{
        //gpio_pin_set(led_dev, APP_ANTSEL_PIN, 0);
		nrf_gpio_pin_set(APP_ANTSEL_PIN);
		printk("Set ANTSEL_PIN(20) high \n");
		//err = bt_set_name("BT840N ANT2");
		temp_name[9] = '2';
    }  	
    if (err) {
        printk("Set Name error:(err %d)\n", err);
    }
#elif BT40
    //err = bt_set_name("BT40");
	//char temp_name[] = {'B','T','4','0',' ',' ',' '};
	char temp_name[] = {'E','V','-','B','T','4','0',' ',' ',' '};
	if (err) {
        printk("BLE set name error (err %d)\n", err);
    }else{
        printk("BLE set name success\n");
    }
#elif BT40N_V6
	//GPIO 1.05 low ANT1 PCB ANT
	//GPIO 1.05 high ANT2 u.fl
    //uint8_t rc = gpio_pin_get(led_dev,5);
	uint8_t rc1 = nrf_gpio_pin_read(5);
	//char temp_name[] = {'B','T','4','0','N','1'};
	char temp_name[] = {'E','V','-','B','T','4','0','N','1'};
	if(rc1 == 0){		
        //gpio_pin_set(led_dev1, 05, 1);
		nrf_gpio_pin_clear(APP_ANTSEL_PIN);
		LOG_INF("Set ANT_SEL(P1.05) low! \n");
		printk("Set ANT_SEL(P1.05) low! \n");

    }else{
        //gpio_pin_set(led_dev1, 05, 0);	
		nrf_gpio_pin_set(APP_ANTSEL_PIN);
		LOG_INF("Set ANT_SEL(P1.05) high! \n");
		printk("Set ANT_SEL(P1.05) high! \n");
		//temp_name[5] = '2';
		temp_name[8] = '2';
    }  	
    if (err) {
        LOG_INF("Set Name error:(err %d)\n", err);
    }
#else
    err = bt_set_name("Unknow device");
#endif

	err = bt_nus_init(&nus_cb);
	if (err) {
		LOG_ERR("Failed to initialize UART service (err: %d)", err);
		return 0;
	}

#ifdef BLE_ADV_NAME

	struct bt_data adname1[] = {
	BT_DATA_BYTES(BT_DATA_FLAGS, (BT_LE_AD_GENERAL | BT_LE_AD_NO_BREDR)),
	BT_DATA(BT_DATA_NAME_COMPLETE, temp_name, sizeof(temp_name)),
	};		

	/* Make a local copy of the BLE advertising parameters */
	struct bt_le_adv_param adv_params = *(BT_LE_ADV_NCONN_IDENTITY); 
	adv_params.interval_min = 0x0020;	//(advertising internal 32*0.625 = 20ms)
	adv_params.interval_max = 0x0040;	//(advertising internal 64*0.625 = 40ms)

	/* Start advertising */
	err = bt_le_adv_start(BT_LE_ADV_CONN, adname1, ARRAY_SIZE(adname1), sd, ARRAY_SIZE(sd));	//advertising internal 100ms
	//err = bt_le_adv_start(&adv_params, adname1, ARRAY_SIZE(adname1), sd, ARRAY_SIZE(sd));
	if (err) {
		printk("Advertising failed to start (err %d)\n", err);
		return;
	}else{
			LOG_INF("Advertising start \n");
		   	printk("Advertising start \n");
    }				

#else
	/*
	err = bt_le_adv_start(BT_LE_ADV_CONN, ad, ARRAY_SIZE(ad), sd,
			      ARRAY_SIZE(sd));
	*/
	err = bt_le_adv_start(BT_LE_ADV_CONN_NAME, ad, ARRAY_SIZE(ad), sd,
			      ARRAY_SIZE(sd));	
	if (err) {
		LOG_ERR("Advertising failed to start (err %d)\n", err);
		printk("Advertising failed to start (err %d)\n", err);
		return 0;
	}else{
		   printk("Advertising start \n");
    }
#endif

#ifdef BT40N_V6
	#if 1
	printk("------------------------------\n");
	uint8_t * cap_xc12 = (uint8_t *) 0x500045C4;
    LOG_INF("0x500045C4 = 0x%x%x%x%x\n",cap_xc12[3],cap_xc12[2],cap_xc12[1],cap_xc12[0]);
	printk("0x500045C4 = 0x%x%x%x%x\n",cap_xc12[3],cap_xc12[2],cap_xc12[1],cap_xc12[0]);

	uint8_t * int_cap = (uint8_t *) 0x500046D0;
    LOG_INF("0x500046D0 = 0x%x%x%x%x\n",int_cap[3],int_cap[2],int_cap[1],int_cap[0]);
	printk("0x500046D0 = 0x%x%x%x%x\n",int_cap[3],int_cap[2],int_cap[1],int_cap[0]);

	#ifdef DCDC_VOLTAGE
	//nrfjprog -f nrf53 --memwr 0x00FF8010 --val 0xFFFFFFFD
	change_gpio_voltage(UICR_VREGHVOUT_VREGHVOUT_3V3);
	uint8_t * int_reg = (uint8_t *) 0x00FF8010;		//VREGHVOUT
    LOG_INF("0x00FF8010 = 0x%x%x%x%x\n",int_reg[3],int_reg[2],int_reg[1],int_reg[0]);
	printk("0x00FF8010 = 0x%x%x%x%x\n",int_reg[3],int_reg[2],int_reg[1],int_reg[0]);

	uint8_t * int_voltge = (uint8_t *) 0x50004B00;		//VREGH.DCDCEN:DC/DC enable register for VREGH
    LOG_INF("0x50004B00 = 0x%x%x%x%x\n",int_voltge[3],int_voltge[2],int_voltge[1],int_voltge[0]);
	printk("0x50004B00 = 0x%x%x%x%x\n",int_voltge[3],int_voltge[2],int_voltge[1],int_voltge[0]);

	uint8_t * int_voltge2 = (uint8_t *) 0x50004704;		//VREGMAIN.DCDCEN:DC/DC enable register for VREGMAIN
    LOG_INF("0x50004704 = 0x%x%x%x%x\n",int_voltge2[3],int_voltge2[2],int_voltge2[1],int_voltge2[0]);
	printk("0x50004704 = 0x%x%x%x%x\n",int_voltge2[3],int_voltge2[2],int_voltge2[1],int_voltge2[0]);
	printk("------------------------------\n");
	#endif
	#endif
#endif
	printk("Test Version:v240_230828 \n");

	for (;;) {
		
		//dk_set_led(RUN_STATUS_LED, (++blink_status) % 2);
		if(command_pending){
            command_pending = false;
            Command_Pro();
        }	

		struct uart_data_t *buf = k_fifo_get(&fifo_uart_rx_data,
						     K_FOREVER);
							 
		bt_nus_send(NULL, buf->data, buf->len);
		//k_sleep(K_MSEC(RUN_LED_BLINK_INTERVAL));	
		k_sleep(K_SECONDS(1));	

	}
}

void ble_write_thread(void)
{
	/* Don't go any further until BLE is initialized */
	k_sem_take(&ble_init_ok, K_FOREVER);

	for (;;) {
		/* Wait indefinitely for data to be sent over bluetooth */
		struct uart_data_t *buf = k_fifo_get(&fifo_uart_rx_data,
						     K_FOREVER);

		if (bt_nus_send(NULL, buf->data, buf->len)) {
			LOG_WRN("Failed to send data over BLE connection");
		}

		LOG_HEXDUMP_INF(buf->data, sizeof(buf->data), "TX.log hexdump info");
		LOG_INF("TX.len = %d\n",buf->len);
		//printk("TX.len = %d\n",buf->len);
		k_free(buf);
	}
}

K_THREAD_DEFINE(ble_write_thread_id, STACKSIZE, ble_write_thread, NULL, NULL,
		NULL, PRIORITY, 0, 0);
